@@ -1,97 +1,4 @@
-#####################
-# Some functions to transform user-provided (i.e., fixed) parameter values from one 
-# parameterization to the other.
-#####################
-beta2eta <-function(first, last, parms.df, out.zero=TRUE, in.zero=FALSE)
-{
-  df.new = parms.df
-  df.new$eta = beta2eta_(df.new$item_score, df.new$beta, first, last)
-  if (out.zero!=in.zero)
-  {
-    if (in.zero) df.new = parms.df[-first,] # zero in but not out
-    if (out.zero) # zero out but not in
-    {
-      tmp = add_zero(parms.df$item_score, parms.df$b, first, last)
-      df.new = data.frame(item_id = rep("i",length(tmp$a)), item_score = tmp$a, 
-                          beta = rep(0,length(tmp$a)), eta = rep(0,length(tmp$a)),
-                          stringsAsFactors = FALSE)
-      for (i in 1:length(tmp$first))
-      {
-        df.new$item_id[tmp$first[i]:tmp$last[i]] = parms.df$item_id[first[i]]
-        df.new$beta[(tmp$first[i]+1):tmp$last[i]] = parms.df$beta[first[i]:last[i]]
-        df.new$eta[(tmp$first[i]+1):tmp$last[i]] = parms.df$eta[first[i]:last[i]]
-      }
-    }
-  }
-  return(df.new)
-}
 
-beta2b <-function(first, last, parms.df, out.zero=TRUE, in.zero=FALSE)
-{
-  df.new = parms.df
-  df.new$b = eta2b_(beta2eta_(df.new$item_score, df.new$beta, first, last))
-  if (out.zero!=in.zero)
-  {
-    if (in.zero) df.new = parms.df[-first,] # zero in but not out
-    if (out.zero) # zero out but not in
-    {
-      tmp = add_zero(df.new$item_score, df.new$b, first, last)
-      df.new = data.frame(item_id = rep("i",length(tmp$a)), item_score = tmp$a, 
-                          beta = rep(0,length(tmp$a)), b = tmp$b,
-                          stringsAsFactors = FALSE)
-      for (i in 1:length(tmp$first))
-      {
-        df.new$item_id[tmp$first[i]:tmp$last[i]] = parms.df$item_id[first[i]]
-        df.new$beta[(tmp$first[i]+1):tmp$last[i]] = parms.df$beta[first[i]:last[i]]
-      }
-    }
-  }
-  return(df.new)
-}
-
-eta2b <-function(first, last, parms.df, out.zero=TRUE, in.zero=FALSE)
-{
-  df.new = parms.df
-  df.new$b = eta2b_(df.new$eta)
-  if (out.zero!=in.zero)
-  {
-    if (in.zero) df.new = parms.df[-first,] # zero in but not out
-    if (out.zero) # zero out but not in
-    {
-      tmp = add_zero(df.new$item_score, df.new$b, first, last)
-      df.new = data.frame(item_id = rep("i",length(tmp$a)), item_score = tmp$a, 
-                          eta = rep(0,length(tmp$a)), b = tmp$b,
-                          stringsAsFactors = FALSE)
-      for (i in 1:length(tmp$first))
-      {
-        df.new$item_id[tmp$first[i]:tmp$last[i]] = parms.df$item_id[first[i]]
-        df.new$eta[(tmp$first[i]+1):tmp$last[i]] = parms.df$eta[first[i]:last[i]]
-      }
-    }
-  }
-  return(df.new)
-}
-
-b2b <-function(first, last, parms.df, out.zero=TRUE, in.zero=TRUE)
-{
-  df.new=parms.df
-  if (out.zero!=in.zero)
-  {
-    if (in.zero) df.new = parms.df[-first,] # zero in but not out
-    if (out.zero) # zero out but not in
-    {
-      tmp = add_zero(parms.df$item_score, parms.df$b, first, last)
-      df.new = data.frame(item_id = rep("i",length(tmp$a)), item_score = tmp$a, 
-                          b = tmp$b, stringsAsFactors = FALSE)
-      for (i in 1:length(tmp$first))
-      {
-        df.new$item_id[tmp$first[i]:tmp$last[i]] = parms.df$item_id[first[i]]
-        df.new$eta[(tmp$first[i]+1):tmp$last[i]] = parms.df$eta[first[i]:last[i]]
-      }
-    }
-  }
-  return(df.new)
-}
 
 # item_id, item_score, and b, eta of beta
 transform.df.parms = function(parms.df, out.format = c('b','beta','eta'), include.zero = TRUE)
@@ -100,6 +7,8 @@ transform.df.parms = function(parms.df, out.format = c('b','beta','eta'), includ
   out.format = match.arg(out.format)
   colnames(parms.df) = tolower(colnames(parms.df))
   
+  if('delta' %in% colnames(parms.df))
+    parms.df = rename(parms.df, beta = 'delta')
   in.format = intersect(colnames(parms.df), c('b','beta','eta'))
   
   if(length(in.format) == 0)
@@ -117,6 +26,9 @@ transform.df.parms = function(parms.df, out.format = c('b','beta','eta'), includ
   if(any(parms.df$item_score%%1 > 0))
     stop("column 'item_score' must be integer valued")
   
+  if(n_distinct(parms.df$item_id, parms.df$item_score) < nrow(parms.df))
+    stop('multiple parameters supplied for the same item and score')
+  
   parms.df = parms.df %>% 
     mutate(item_id = as.character(.data$item_id), item_score = as.integer(.data$item_score)) %>%
     arrange(.data$item_id, .data$item_score)
@@ -130,7 +42,7 @@ transform.df.parms = function(parms.df, out.format = c('b','beta','eta'), includ
   in.zero = any(mm$min_score == 0)
   
   if(in.zero && any(mm$min_score) != 0)
-    stop("Either all item paramters should include a zero score parameter or none should")
+    stop("Either all items or none of the items should include a zero score parameter")
   
   if(any(mm$max_score == 0))
     stop('All items should contain at least one non-zero score parameter')
@@ -144,7 +56,7 @@ transform.df.parms = function(parms.df, out.format = c('b','beta','eta'), includ
   fl = parms.df %>%
     mutate(rn = row_number()) %>%
     group_by(.data$item_id) %>% 
-    summarize(first = min(.data$rn), last=max(.data$rn)) %>%
+    summarize(first = as.integer(min(.data$rn)), last = as.integer(max(.data$rn))) %>%
     ungroup()
   
   args = list(first = fl$first, last = fl$last, parms.df = parms.df, 
@@ -153,8 +65,6 @@ transform.df.parms = function(parms.df, out.format = c('b','beta','eta'), includ
 }
 
 
-
-
 ##########################################
 #' Fit the extended nominal response model
 #'
@@ -162,16 +72,26 @@ transform.df.parms = function(parms.df, out.format = c('b','beta','eta'), includ
 #' or a Gibbs sampler for Bayesian estimation.
 #'
 #'
-#' @param dataSrc Data source: a dexter project db handle or a data.frame with columns: person_id, item_id, item_score
+#' @param dataSrc a connection to a dexter database, a matrix, or a data.frame with columns: person_id, item_id, item_score
 #' @param predicate An optional expression to subset data, if NULL all data is used
 #' @param fixed_params Optionally, a prms object from a previous analysis or 
-#' a data.frame with columns: item_id, item_score (omitting 0 score category) and beta. To facilitate the user in entering parameter values, we assume the parameterisation used by OPLM; in short, beta's are thresholds between categories. At this moment, it is not possible to fix some but not all categories of an item.
+#' a data.frame with parameters, see details.
 #' @param method If CML, the estimation method will be Conditional Maximum Likelihood;
 #' otherwise, a Gibbs sampler will be used to produce a sample from the posterior
 #' @param nIterations Number of Gibbs samples when estimation method is Bayes. The maximum 
 #' number of iterations when using CML.
+#' @param merge_within_persons whether to merge different booklets administered to the same person, enabling linking over persons as well as booklets.
 #' @return An object of type \code{prms}. The prms object can be cast to a data.frame of item parameters 
 #' using function `coef` or used directly as input for other Dexter functions.
+#' @details
+#' To support some flexibility in fixing parameters, fixed_params can be a dexter prms object of a data.frame.
+#' If a data.frame, it should contain the columns item_id, item_score and a difficulty parameter. Three types of parameters are supported:
+#' \describe{
+#' \item{delta/beta}{ thresholds between subsequent item categories }
+#' \item{eta}{item-category parameters} 
+#' \item{b}{exp(-eta)}
+#' }
+#' Each type corresponds to a different parametrization of the model. 
 #' 
 #' @references 
 #' Maris, G., Bechger, T.M. and San-Martin, E. (2015) A Gibbs sampler for the (extended) marginal Rasch model. 
@@ -180,51 +100,63 @@ transform.df.parms = function(parms.df, out.format = c('b','beta','eta'), includ
 #' @seealso functions that accept a prms object as input: \code{\link{ability}}, \code{\link{plausible_values}}, 
 #' \code{\link{plot.prms}}
 #'
-fit_enorm = function(dataSrc, predicate = NULL, fixed_params = NULL, method=c("CML", "Bayes"), nIterations=500)
+fit_enorm = function(dataSrc, predicate = NULL, fixed_params = NULL, method=c("CML", "Bayes"), 
+                     nIterations=500, merge_within_persons=FALSE)
 {
   method = match.arg(method)
+  check_dataSrc(dataSrc)
+  check_num(nIterations, 'integer', .length=1, .min=1)
   qtpredicate = eval(substitute(quote(predicate)))
-  check_arg(dataSrc, 'dataSrc')
-  check_arg(nIterations, 'integer', .length=1)
-  env=caller_env()
-  fit_enorm_(dataSrc, qtpredicate = qtpredicate, fixed_params = fixed_params, method=method, nIterations=nIterations, env=env)
+  env = caller_env()
+  fit_enorm_(dataSrc, qtpredicate = qtpredicate, fixed_params = fixed_params, 
+             method=method, nIterations=nIterations, env=env, merge_within_persons=merge_within_persons)
 }
 
 
-fit_enorm_ = function(dataSrc, qtpredicate = NULL, fixed_params = NULL, method=c("CML", "Bayes"), nIterations=500, env=NULL) 
+fit_enorm_ = function(dataSrc, qtpredicate = NULL, fixed_params = NULL, method=c("CML", "Bayes"), 
+                      nIterations=500, env=NULL, merge_within_persons=FALSE) 
 {
   method = match.arg(method)
   if(is.null(env)) env = caller_env()
   
-  r = get_resp_data(dataSrc, qtpredicate, summarised=FALSE, env=env)
+  respData = get_resp_data(dataSrc, qtpredicate, summarised=FALSE, env=env, retain_person_id=FALSE,
+                           merge_within_persons = merge_within_persons)
   
-  x = r$x
-  design = r$design
-  if(nrow(x) == 0) stop('no data to analyse')
-  if(nrow(design) == 1) stop('There are responses to only one item in your selection, this cannot be calibrated.') 
+  design = respData$design
+  
+  # to~do: I think we need tests with at least three items, fischer criterium
+  if(nrow(design) == 1) 
+    stop('There are responses to only one item in your selection, this cannot be calibrated.') 
   
   if(!is_connected(design))
     stop('Your design is not connected')  
 
-  itm_max = x %>% 
+  ss = get_sufStats_nrm(respData)
+  ssIS = ss$ssIS
+  plt = ss$plt
+  
+  # bug in dplyr, min/max of integer in group_by becomes double
+  ssI  = ssIS %>% 
+    mutate(rn = row_number()) %>%
+    group_by(.data$item_id) %>%
+    summarise(first = as.integer(min(.data$rn)),last = as.integer(max(.data$rn))) %>%
+    ungroup() %>%
+    arrange(.data$item_id)
+  
+  if(any(ssI$first == ssI$last)) 
+  {
+  	message('Items without score variation:')
+  	print(as.character(ssI$item_id[ssI$first == ssI$last]))
+  	stop('One or more items are without score variation')
+  }
+  
+  design = design %>%
+    inner_join(ssI,by='item_id') %>%
+    arrange(.data$booklet_id, .data$first)
+  
+  itm_max = ssIS %>% 
     group_by(.data$item_id) %>% 
-    summarise(maxScore = max(.data$item_score)) %>% #, nsc = n_distinct(.data$item_score)) %>%
-	  ungroup()
-  
-  if(any(itm_max$maxScore == 0)) 
-    stop('One or more items has a maximum score of 0')
-  # for now, just error. May possibly become warning later
-  
-
-  ssBIS = x %>% 
-    group_by(.data$booklet_id, .data$item_id, .data$item_score) %>% 
-    summarise(sufI=n(), sufC=sum(.data$item_score * .data$sumScore)) %>% 
-    ungroup()
-  
-
-  plt = x %>% 
-    group_by(.data$booklet_id, .data$item_id, .data$sumScore) %>% 
-    summarise(meanScore=mean(.data$item_score), N=n()) %>% 
+    summarise(maxScore = as.integer(max(.data$item_score))) %>% 
     ungroup()
   
   # max booklet scores
@@ -234,75 +166,41 @@ fit_enorm_ = function(dataSrc, qtpredicate = NULL, fixed_params = NULL, method=c
     summarise(maxTotScore = sum(.data$maxScore))
   
   # booklets 0:maxscore
-  allScores = maxScores %>% 
+  all_scores = maxScores %>% 
     group_by(.data$booklet_id) %>%
-    do(tibble(sumScore=0:.$maxTotScore)) %>%
+    do({tibble(booklet_score=0:.$maxTotScore)}) %>%
     ungroup()
   
-
-  stb = distinct(x,.data$person_id,.data$booklet_id,.keep_all=T) %>%
-    group_by(.data$booklet_id, .data$sumScore) %>%
-    summarize(N=n()) %>%
-    right_join(allScores, by=c('booklet_id','sumScore')) %>%
+  scoretab = plt %>%
+    distinct(.data$booklet_id, .data$booklet_score,.data$N) %>%
+    right_join(all_scores, by=c('booklet_id','booklet_score')) %>%
     mutate(N=coalesce(.data$N, 0L)) %>%
-    arrange(.data$booklet_id, .data$sumScore)
+    arrange(.data$booklet_id, .data$booklet_score)
 
-  ssIS = ssBIS %>% 
-    group_by(.data$item_id, .data$item_score) %>%
-    summarise(sufI = sum(.data$sufI)) %>%
-    ungroup() %>%
-    arrange(.data$item_id, .data$item_score)
-  
-  ssI  = ssIS %>% 
-    group_by(.data$item_id) %>%
-    summarise(nCat = n()) %>% 
-    mutate(first = cumsum(.data$nCat) - .data$nCat + 1L,last = cumsum(.data$nCat)) %>%
-    ungroup() %>%
-    arrange(.data$item_id)
-  
-  if(any(ssI$nCat == 1)) stop('One or more items are without score variation')
-  
-  design = design %>%
-    inner_join(ssI,by='item_id') %>%
-    arrange(.data$booklet_id, .data$first)
-  
-  a = ssIS$item_score
-  
-  bkl = lapply(split(design,design$booklet_id),
-    function(bkd)
-    {
-        booklet_id = bkd$booklet_id[1]
-        scoretab = stb$N[stb$booklet_id==booklet_id]
-        list(booklet_id = booklet_id,
-             first = bkd$first,
-             last = bkd$last,
-             scoretab = scoretab,
-             m = sum(scoretab),
-             lambda = rep(1,length(scoretab)))
-    })
 
-  
-  #it_sc_lab = paste0(ssIS$item_id[-ssI$first], "_",ssIS$item_score[-ssI$first])
   fixed_b = NULL
   has_fixed_parms = !is.null(fixed_params)
-  
+
   ## deal with fixed parameters
   if(has_fixed_parms)
   {
     if(inherits(fixed_params,'prms'))
     {
       if (fixed_params$inputs$method!="CML")
-        message("Posterior means are taken as values of fixed parameters")
+        message("Posterior means are taken as values for fixed parameters")
       
       fixed_params = fixed_params$inputs$ssIS %>%
-        add_column(b = if.else(fixed_params$inputs$method=="CML", fixed_params$est$b, colMeans(fixed_params$est$b))) %>%
-        right_join(ssIS, by=c('item_id','item_score')) %>%
-        arrange(.data$item_id,.data$item_score) 
+        add_column(b = if.else(fixed_params$inputs$method=="CML", fixed_params$est$b, colMeans(fixed_params$est$b)))
+      
     } else
     {
       # transform the fixed params to the b parametrization dexter uses internally
       fixed_params = transform.df.parms(fixed_params, out.format = 'b', include.zero = TRUE) 
     }  
+    # avoid factor warnings and reduce
+    fixed_params$item_id = ffactor(as.character(fixed_params$item_id), levels=levels(design$item_id))
+    fixed_params = filter(fixed_params,!is.na(.data$item_id)) 
+    
     # check for missing categories in fixed_params, necessary?
     missing_cat = ssIS %>% 
       semi_join(fixed_params, by='item_id') %>%
@@ -326,43 +224,33 @@ fit_enorm_ = function(dataSrc, qtpredicate = NULL, fixed_params = NULL, method=c
       arrange(.data$item_id,.data$item_score) %>%
       pull(.data$b)
     
-    # this test fails for some reason
     if(!anyNA(fixed_b)) stop('nothing to calibrate, all parameters are fixed')
   }
   
   if (method=="CML"){
-    result = try(calibrate_CML(booklet=bkl, sufI=ssIS$sufI, a=ssIS$item_score, 
+    result = calibrate_CML(scoretab=scoretab, design=design, sufI=ssIS$sufI, a=ssIS$item_score, 
                                first=ssI$first, last=ssI$last, nIter=nIterations,
-                               fixed_b=fixed_b))
-    #names(result$b)= paste0(ssIS$item_id, "_",ssIS$item_score)
-    #row.names(result$beta)=it_sc_lab
-    #rownames(result$acov.beta)=it_sc_lab
-    #colnames(result$acov.beta)=it_sc_lab
+                               fixed_b=fixed_b)
   } else 
   {
-    design = design %>%
-      mutate(bn=dense_rank(.data$booklet_id)) %>%
-      arrange(.data$first, .data$bn)
-    
-    itemListInt = split(design$bn, design$item_id)
-    
-    b = exp(runif(nrow(ssIS), -1, 1))
-    
-    result = calibrate_Bayes(itemList=itemListInt, booklet=bkl, sufI=ssIS$sufI, b=b, a=a, 
-                                 first=ssI$first, last=ssI$last, nIter=nIterations,
-                                 fixed_b=fixed_b)
-    #colnames(result$b)= paste0(ssIS$item_id, "_",ssIS$item_score) 
-    #colnames(result$beta)=it_sc_lab
+    result = calibrate_Bayes(scoretab=scoretab, design=design, sufI=ssIS$sufI, a=ssIS$item_score,
+                              b=exp(runif(nrow(ssIS), -1, 1)), 
+                              first=ssI$first, last=ssI$last, nIter=nIterations, fixed_b=fixed_b)
   }
-  if (inherits(result, "try-error")) stop('Calibration failed')
   
-  abl_tables = new.env(parent = emptyenv())
-  abl_tables$mle = NULL
+  mle = design %>% 
+    group_by(.data$booklet_id) %>%
+    do({
+      est = theta_MLE(result$b, a=ssIS$item_score, .$first, .$last, se=FALSE)
+      theta = est$theta[2:(length(est$theta)-1)]
+      tibble(booklet_score=1:length(theta), theta = theta)
+    }) %>%
+    ungroup() 
   
   outpt = list(est=result, 
-               inputs=list(bkList=bkl, ssIS=ssIS, ssI=ssI, design=r$design,plt=plt,
+               inputs=list(scoretab=scoretab, design=design, ssIS=ssIS, ssI=ssI, design=design,plt=plt,
                             method=method, has_fixed_parms = has_fixed_parms), 
-               abl_tables = abl_tables,
+               abl_tables = list(mle = mle),
                xpr=deparse(qtpredicate))
   class(outpt) = append('prms', class(outpt)) 
   outpt
@@ -370,7 +258,7 @@ fit_enorm_ = function(dataSrc, qtpredicate = NULL, fixed_params = NULL, method=c
 
 
 
-
+# to~do: is there a better plot for calibrate bayes?
 #' Plot for the extended nominal Response model
 #' 
 #' The plot shows 'fit' by comparing the expected score based on the model (grey line)
@@ -379,33 +267,62 @@ fit_enorm_ = function(dataSrc, qtpredicate = NULL, fixed_params = NULL, method=c
 #' 
 #' @param x object produced by fit_enorm
 #' @param item_id which item to plot, if NULL, one plot for each item is made
+#' @param dataSrc data source, see details
+#' @param predicate an expression to subset data in dataSrc
 #' @param nbins number of ability groups
 #' @param ci confidence interval for the error bars, between 0 and 1. 0 means no error bars.
 #' Default = 0.95 for a 95\% confidence interval
 #' @param ... further arguments to plot
 #' 
+#' @details
+#' The standard plot shows the fit against the sample on which the parameters were fitted. If
+#' dataSrc is provided, the fit is shown against the observed data in dataSrc. This may be useful 
+#' for plotting the fit in different subgroups as a visual test for item level DIF.
+#' 
 #' @method plot prms
 #' 
-plot.prms = function(x,item_id=NULL, nbins=5, ci = .95, ...)
+plot.prms = function(x, item_id=NULL, dataSrc=NULL, predicate=NULL, nbins=5, ci = .95, ...)
 {
-  if(is.null(x$inputs$plt))
-  {
-    if(inherits(x,'mst_enorm')) stop('Sorry, the plot method for enorm_mst will only be supported in dexterMST from version 0.1.1 onwards')
-    stop('Sorry, the plot method is only available for parameter objects produced with dexter 0.8.1 or later')
-  }
+  check_num(nbins,'integer',.length=1, .min=2)
   
   # if no item id provided plot them all
   if(is.null(item_id))
     item_id = x$inputs$ssI$item_id
 
-  if(is.null(x$abl_tables$mle))
-    x$abl_tables$mle = ability_tables(x, standard_errors=FALSE, method='MLE')
+  if(!is.null(dataSrc))
+  {
+    check_dataSrc(dataSrc)
+    qtpredicate = eval(substitute(quote(predicate)))
+    env = caller_env()
+    respData = get_resp_data(dataSrc, qtpredicate, env=env, retain_person_id=FALSE,
+                             parms_check=filter(x$inputs$ssIS, .data$item_id %in% local(item_id)))
+    
+    x$abl_tables = list() # in dexmst it was briefly an environment
+    
+    x$abl_tables$mle = respData$design %>% 
+      inner_join(x$inputs$ssI,by='item_id') %>%
+      group_by(.data$booklet_id) %>%
+      do({
+        est = theta_MLE(x$est$b, a=x$inputs$ssIS$item_score, .$first, .$last, se=FALSE)
+        theta = est$theta[2:(length(est$theta)-1)]
+        tibble(booklet_score=1:length(theta), theta = theta)
+      }) %>%
+      ungroup() 
+    
+    x$inputs$plt = get_sufStats_nrm(respData)$plt
+  }
   
+  #old dexterMST
+  if(is.null(x$abl_tables$mle))
+  {
+    x$abl_tables$mle = ability_tables(x,method='MLE', standard_errors=FALSE) %>%
+      filter(is.finite(.data$theta))
+  }
+  #many plots
   if(length(item_id) > 1)
   {
-    for(item in item_id) plot(x, item_id=item, nbins=nbins, ci=ci, ...)
-    
-    return(invisible(NULL))
+    return(invisible(
+      lapply(item_id, function(itm) plot(x, itm, nbins=nbins, ci=ci, ...))))
   }
   # for dplyr
   item_id_ = item_id
@@ -419,8 +336,7 @@ plot.prms = function(x,item_id=NULL, nbins=5, ci = .95, ...)
   
   plt = x$inputs$plt %>%
     filter(.data$item_id==item_id_) %>%
-    inner_join(x$abl_tables$mle, by=c('booklet_id','sumScore')) %>%
-    filter(is.finite(.data$theta)) %>%
+    inner_join(x$abl_tables$mle, by=c('booklet_id','booklet_score')) %>%
     mutate(abgroup = weighted_ntile(.data$theta, .data$N, n = nbins)) %>%
     group_by(.data$abgroup) %>%
     summarize(gr_theta = weighted.mean(.data$theta,.data$N), avg_score = weighted.mean(.data$meanScore,.data$N), n=sum(.data$N)) %>%
@@ -441,12 +357,13 @@ plot.prms = function(x,item_id=NULL, nbins=5, ci = .95, ...)
   do.call(plot, plot.args)
   lines(plt$gr_theta,plt$expected_score, col='grey80') 
   
+  plt$outlier = FALSE
+  
   if(!is.null(ci) && !is.na(ci) && ci !=0)
   {
-    if(ci>1)
-    {
-      if(ci<100) ci = ci/100
-    }  
+    if(ci>1 && ci<100)
+      ci = ci/100
+    
     if(ci<0 || ci >= 1)
       stop('confidence interval must be between 0 and 1')
     
@@ -455,14 +372,19 @@ plot.prms = function(x,item_id=NULL, nbins=5, ci = .95, ...)
     cmin = function(p, n) pmax(0, p - qnt * sqrt(p*(1-p)/n))
     cmax = function(p, n) pmin(1, p + qnt * sqrt(p*(1-p)/n))
     
-    arrows(plt$gr_theta, max_score*cmin(plt$expected_score/max_score, plt$n), 
-           plt$gr_theta, max_score*cmax(plt$expected_score/max_score, plt$n), 
+    plt = plt %>%
+      mutate(conf_min = max_score * cmin(.data$expected_score/max_score, .data$n),
+             conf_max = max_score * cmax(.data$expected_score/max_score, .data$n)) %>%
+      mutate(outlier = .data$avg_score < .data$conf_min | .data$avg_score > .data$conf_max)
+    
+    arrows(plt$gr_theta, plt$conf_min, 
+           plt$gr_theta, plt$conf_max, 
            length=0.05, angle=90, code=3, col='grey80')
-  }
+  } 
   
   lines(plt$gr_theta,plt$avg_score)  
-  points(plt$gr_theta, plt$avg_score)
-  invisible(NULL)
+  points(plt$gr_theta, plt$avg_score, bg = if_else(plt$outlier, qcolors(1), 'transparent'), pch=21)
+  invisible(df_format(plt))
 }
 
 
@@ -486,7 +408,7 @@ print.prms = function(x, ...){
 #' extract enorm item parameters
 #' 
 #' @param object an enorm parameters object, generated by the function \code{\link{fit_enorm}}
-#' @param bayes_hpd_b width of Bayesian highest posterior density interval around mean_beta, 
+#' @param hpd width of Bayesian highest posterior density interval around mean_beta, 
 #'  value must be between 0 and 1, default is 0.95 
 #' @param ... further arguments to coef are ignored
 #'  
@@ -499,58 +421,43 @@ print.prms = function(x, ...){
 #' 
 #' 
 #' 
-coef.prms = function(object, bayes_hpd_b = 0.95, ...)
+coef.prms = function(object, hpd = 0.95, ...)
 {
-
-  
-  if(bayes_hpd_b <= 0 ||  bayes_hpd_b >= 1)
-    stop('args$bayes_hpd_b must be between 0 and 1')
-  
   x = object
   
-  hpd=function(x, conf=bayes_hpd_b){
-    conf <- min(conf, 1-conf)
-    n <- length(x)
-    nn <- round( n*conf )
-    x <- sort(x)
-    xx <- x[ (n-nn+1):n ] - x[1:nn]
-    m <- min(xx)
-    nnn <- which(xx==m)[1]
-    return(c(l=x[ nnn ],r=x[ n-nn+nnn ]))
-  }
-  
   if (x$inputs$method=="CML")
   {
     atab=data.frame(item_id=x$inputs$ssIS$item_id[-x$inputs$ssI$first],
                     item_score=as.integer(x$inputs$ssIS$item_score[-x$inputs$ssI$first]),
                     beta=x$est$beta,
                     SE_beta=sqrt(diag(x$est$acov.beta)),stringsAsFactors=FALSE)
-  }
-  
-  if (x$inputs$method=="Bayes"){
+  } else
+  {
+    if(hpd <= 0 ||  hpd >= 1)
+      stop('hpd must be between 0 and 1')
     
-    
-    hh = t(apply(x$est$beta,2,hpd))
+    hh = t(apply(x$est$beta,2,hpdens, conf=hpd))
     atab=data.frame(item_id = x$inputs$ssIS$item_id[-x$inputs$ssI$first],
                     a = x$inputs$ssIS$item_score[-x$inputs$ssI$first],
                     mb = colMeans(x$est$beta),
                     sdb = apply(x$est$beta, 2, sd),
                     hpdl = hh[,1], hpdr=hh[,2],stringsAsFactors=FALSE)
     colnames(atab)=c("item_id" ,"item_score", "mean_beta", "SD_beta", 
-                     sprintf("%i_hpd_b_left", round(100 * bayes_hpd_b)),
-                     sprintf("%i_hpd_b_right", round(100 * bayes_hpd_b)))
+                     sprintf("%i_hpd_b_left", round(100 * hpd)),
+                     sprintf("%i_hpd_b_right", round(100 * hpd)))
   }
+  atab$item_id = as.character(atab$item_id)
   rownames(atab) = NULL
-  return(atab)
+  df_format(atab)
 }
 
 
 #' Functions of theta
 #' 
-#' returns information function, expected score function or score simulation function 
-#' for a single item, an arbitrary group of items or all items
+#' returns information function, expected score function or score simulation function for a single item, an arbitrary group of items or all items
 #' 
-#' @param parms object produced by fit_enorm
+#' @param parms object produced by \code{\link{fit_enorm}} or a data.frame with columns item_id, item_score and, 
+#' depending on parametrization, a column named beta/delta, eta or b
 #' @param items vector of one or more item_id's. If NULL and booklet_id is also NULL, all items in parms are used
 #' @param booklet_id id of a single booklet (e.g. the test information function), if items is not NULL this is ignored
 #' @param which.draw the number of the random draw (only applicable if calibration method was Bayes). If NULL, the mean 
@@ -563,6 +470,7 @@ coef.prms = function(object, bayes_hpd_b = 0.95, ...)
 #' \item{r_score}{a matrix with length(theta) rows and one column for each item containing simulated scores based on theta. 
 #' To obtain test scores, use rowSums on this matrix}
 #' }
+#' 
 #' @examples
 #' 
 #' db = start_new_project(verbAggrRules,':memory:')
@@ -588,7 +496,7 @@ coef.prms = function(object, bayes_hpd_b = 0.95, ...)
 #' 
 #' par(new=TRUE)
 #' 
-#' plot(density(pv$PV1), col='green', axes=FALSE,xlab=NA, ylab=NA,main=NA)
+#' plot(density(pv$PV1), col='green', axes=FALSE, xlab=NA, ylab=NA, main=NA)
 #' axis(side=4)
 #' mtext(side = 4, line = 2.5, 'population density (green)')
 #' 
@@ -613,65 +521,90 @@ r_score = function(parms, items=NULL, booklet_id=NULL, which.draw=NULL)
 }
 
 
-# TO DO: should be able to accept a data.frame as wel as parms in the future
-# escore can be more efficient with the main loop in C
-theta_function = function(parms, items=NULL, booklet=NULL, which.draw=NULL, what=c('information','expected','sim'))
+
+
+theta_function = function(parms, items=NULL, booklet=NULL, which.draw=NULL, 
+                          what=c('information','expected','sim'))
 {
   what = match.arg(what)
-  if(is.null(items) && is.null(booklet))
-  {
-    fl = parms$inputs$ssI
-  } else if(!is.null(items))
-  {
-    items = unique(items)
-    
-    if(length(setdiff(items,parms$inputs$ssI$item_id))>0)
-    {
-      message('unknown items:')
-      print(sort(setdiff(items,parms$inputs$ssI$item_id)))
-      stop('Some items were not found, see output')
-    }
-    
-    fl = parms$inputs$ssI %>% 
-      semi_join(tibble(item_id=items),by='item_id') %>% 
-      arrange(.data$first)
-    
-  } else
-  {
-    booklet = as.character(booklet)
-    check_arg(booklet,'character',.length=1)
-    
-    fl = parms$inputs$design %>%
-      filter(.data$booklet_id==booklet) %>%
-      inner_join(parms$inputs$ssI, by='item_id') %>%
-      arrange(.data$first)
-    if(nrow(fl)==0)
-      stop(paste0('booklet `',booklet,'` not found'))
-  }
   
-  if(is.matrix(parms$est$b))
+  # data preparation
+  # create fl(item_id,first,last), a, b
+  
+  if(inherits(parms,'data.frame'))
   {
-    if(is.null(which.draw))
+    out = transform.df.parms(parms,'b',TRUE)
+    a = out$item_score
+    b = out$b
+    
+    fl = out %>%
+      mutate(rn=row_number()) %>%
+      group_by(.data$item_id) %>%
+      summarize(first=as.integer(min(.data$rn)), last=as.integer(max(.data$rn))) %>%
+      ungroup()
+    
+    if(!is.null(items))
+      fl = filter(fl, fl$item_id %in% items)
+    
+  } else if(inherits(parms,'prms'))
+  {
+    a = parms$inputs$ssIS$item_score
+    b = parms$est$b
+    if(is.matrix(b))
     {
-      parms$est$b = colMeans(parms$est$b)
-    } else
-    {
-      if(which.draw<1 || which.draw > nrow(parms$est$b))
-        stop('parameter `which.draw` out of range')
-      parms$est$b = as.vector(parms$est$b[which.draw,])
+      if(is.null(which.draw))
+      {
+        b = colMeans(b)
+      } else
+      {
+        if(which.draw<1 || which.draw > nrow(b))
+          stop('argument `which.draw` out of range')
+        b = as.vector(b[which.draw,])
+      }
     }
-  }
+    
+    fl = parms$inputs$ssI
+    fl$item_id = as.character(fl$item_id)
+    
+    if(!is.null(items))
+    {
+      items = unique(items)
+      suppressWarnings({fl = semi_join(fl, tibble(item_id=items),by='item_id')})
+      if(nrow(fl) != length(items))
+      {
+        message('unknown items:')
+        print(sort(setdiff(items,fl$item_id)))
+        stop('Some items were not found, see output')
+      }
+    } else if(!is.null(booklet))
+    {
+      booklet = unique(booklet)
+      design = parms$inputs$design
+      if(length(intersect(booklet,design$booklet_id))<length(booklet))
+      {
+        stop('unknown booklet')
+      }
+      
+      fl = design %>%
+        filter(.data$booklet_id %in% booklet) %>%
+        distinct(.data$item_id, .keep_all=TRUE)
+    }  
+    fl = arrange(fl,.data$first)
+  }  
+  rm(parms)  
+  #output
+  
   if(what=='information')
   {
     out = function(theta)
     {
-      check_arg(theta,'numeric')
+      check_num(theta)
       if(any(is.na(theta) | is.nan(theta)))
         stop('theta may not contain nan/NA values')  
       
       res = rep(0,length(theta))
       res[is.finite(theta)] = 
-        IJ_(parms$est$b,parms$inputs$ssIS$item_score,fl$first, fl$last, theta[is.finite(theta)])$I
+        IJ_(b,a,fl$first, fl$last, theta[is.finite(theta)])$I
       # extremely large values overflow to NaN, recover as 0
       res[is.nan(res)] = 0
       res
@@ -680,17 +613,18 @@ theta_function = function(parms, items=NULL, booklet=NULL, which.draw=NULL, what
     
   } else if(what == 'expected')
   {
-    max_score = sum(parms$inputs$ssIS$item_score[fl$last])
+    max_score = sum(a[fl$last])
     out = function(theta)
     {
-      check_arg(theta,'numeric')
+      check_num(theta)
       if(any(is.na(theta) | is.nan(theta))) 
         stop('theta may not contain nan/NA values') 
       
       res = rep(0,length(theta))
       
-      res[is.finite(theta)] =
-        sapply(theta[is.finite(theta)], E_score, b=parms$est$b, a=parms$inputs$ssIS$item_score, first=fl$first, last=fl$last)
+      res[is.finite(theta)] = E_score(theta[is.finite(theta)],  
+                                      b=b, a=a, 
+                                      first=fl$first, last=fl$last)
       
       res[is.infinite(theta) & theta > 0] = max_score
       # extremely large values of theta overflow to NaN (small values undeflow to zero, which is fine)
@@ -702,11 +636,10 @@ theta_function = function(parms, items=NULL, booklet=NULL, which.draw=NULL, what
   {
     out = function(theta)
     {
-      res = mapply(rscore, first = fl$first, last = fl$last,
-                     MoreArgs = list(theta = theta, b = parms$est$b, a = parms$inputs$ssIS$item_score))
+      res = rscore_item(theta,b=b,a=a,first = fl$first, last = fl$last)
       colnames(res) = fl$item_id
       res
-
+      
     }
     class(out) = append('sim_func',class(out))
   }
@@ -715,6 +648,7 @@ theta_function = function(parms, items=NULL, booklet=NULL, which.draw=NULL, what
 }
 
 print.inf_func = function(x,...) cat('Information function I(theta)\n')
-print.exp_func = function(x,...) cat('Expected score function E(theta)\n')
-print.sim_func = function(x,...)  cat('(x_i1, ..., x_in) ~ ENORM([theta])\n\tfunction (theta)\n')
+print.exp_func = function(x,...) cat('Conditional expected score function E(X|theta)\n')
+print.sim_func = function(x,...) cat('(x_i1, ..., x_in) ~ ENORM([theta])\n\tfunction (theta)\n')
+
 
